@@ -36,7 +36,7 @@ class WebSocketWantReadError(ssl.SSLWantReadError):
 class WebSocketWantWriteError(ssl.SSLWantWriteError):
     pass
 
-class WebSocket(object):
+class WebSocket:
     """WebSocket protocol socket like class.
 
     This provides access to the WebSocket protocol by behaving much
@@ -139,7 +139,9 @@ class WebSocket(object):
             self.socket = socket.create_connection((uri.hostname, port))
 
             if uri.scheme in ("wss", "https"):
-                self.socket = ssl.wrap_socket(self.socket)
+                context = ssl.create_default_context()
+                self.socket = context.wrap_socket(self.socket,
+                                                  server_hostname=uri.hostname)
                 self._state = "ssl_handshake"
             else:
                 self._state = "headers"
@@ -666,7 +668,7 @@ class WebSocket(object):
                             continue
 
                 if code is None:
-                    self.close_code = code = 1005
+                    self.close_code = 1005
                     self.close_reason = "No close status code specified by peer"
                 else:
                     self.close_code = code
@@ -731,7 +733,7 @@ class WebSocket(object):
         if self.client:
             mask = b''
             for i in range(4):
-                mask += random.randrange(256)
+                mask += random.randrange(256).to_bytes()
             frame = self._encode_hybi(opcode, msg, mask)
         else:
             frame = self._encode_hybi(opcode, msg)
